package com.cicdi.notify.sms.telecom;

import com.alibaba.fastjson.JSONObject;
import com.cicdi.notify.*;
import com.cicdi.notify.sms.SmsProvider;
import com.cicdi.notify.template.TemplateManager;
import com.cicdi.notify.util.ExpressionUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

/**
 * 云信通短信通知平台通知器
 *
 * @author xueye
 */
public class TelecomSmsNotifier implements Notifier<TelecomSmsTemplate> {

    private static final String notifyApi = "http://www.js139.com.cn:8022/hysms/SendMsg";

    private final TelecomSmsNotifierConfiguration configuration;

    private final String notifierId;

    private final CloseableHttpClient httpClient;

    public TelecomSmsNotifier(NotifierProperties properties) {
        this.httpClient = HttpClientBuilder.create().build();
        this.notifierId = properties.getId();
        this.configuration = new JSONObject(properties.getConfiguration()).toJavaObject(TelecomSmsNotifierConfiguration.class);
    }

    @Override
    public String getNotifierId() {
        return notifierId;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.sms;
    }

    @Override
    public Provider getProvider() {
        return SmsProvider.js139;
    }

    @Override
    public void send(TelecomSmsTemplate template, Map<String, Object> context) {
        try {
            URI sendMsgURI = new URIBuilder(notifyApi).addParameter("userId", configuration.getUserId())
                    .addParameter("password", configuration.getPassword())
                    .addParameter("content", render(template.getContent(), context))
                    .addParameter("mobile", template.getMobile())
                    .build();
            HttpPost httpPost = new HttpPost(sendMsgURI);
            CloseableHttpResponse response = httpClient.execute(httpPost);
            String responseString = EntityUtils.toString(response.getEntity());
            Map<String, String> resultMap = handleResponse(responseString);
            String resultCodeKey = "rspCode";
            if (resultMap.containsKey(resultCodeKey)) {
                if (!"0".equals(resultMap.get(resultCodeKey)) && !"DELIVRD".equalsIgnoreCase(resultMap.get(resultCodeKey))) {
                    throw new RuntimeException("短信发送失败：" + resultMap.get("rspDesc"));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void close() {
        try {
            httpClient.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 使用变量替和SpEL表达式渲染模板
     *
     * @param expression 带有变量或者表达式的模板
     * @param context    用于替换的上下文
     * @return 渲染后的模板
     */
    private String render(String expression, Map<String, Object> context) {
        return ExpressionUtils.analytical(expression, context);
    }

    /**
     * 处理HTTP请求响应信息
     * <p>将URL参数转换成键值对</p>
     *
     * @param response 响应字符串
     * @return 处理过的响应信息
     */
    private Map<String, String> handleResponse(String response) {
        Map<String, String> map = new HashMap<>();
        String[] param = response.split("&");
        for (String key : param) {
            String[] pair = key.split("=");
            if (pair.length == 2) {
                map.put(pair[0], pair[1]);
            }
        }
        return map;
    }
}
